@@ -61,13 +61,17 @@
 
 #include "app-internal.h"
 
+#define GCC_VERSION (__GNUC__ * 10000 \
+			+ __GNUC_MINOR__ * 100 \
+			+ __GNUC_PATCHLEVEL__)
+
 struct hash_name {
 	const char *kcapiname;
 	const char *bsdname;
 };
 
 struct hash_key {
-	const char *subdir;
+	const char *checkdir;
 	const uint8_t *data;
 	uint32_t len;
 };
@@ -104,12 +108,20 @@ static const char hmaccalc_hmackey[] = "FIPS-FTW-RHT2009";
 static const struct hash_key KEY_FIPSCHECK = {
 	.data = (const uint8_t *)fipscheck_hmackey,
 	.len = sizeof(fipscheck_hmackey) - 1,
-	.subdir = "fipscheck",
+#ifdef CHECK_DIR
+	.checkdir = CHECK_DIR"/fipscheck",
+#else
+	.checkdir = NULL,
+#endif
 };
 static const struct hash_key KEY_HMACCALC = {
 	.data = (const uint8_t *)hmaccalc_hmackey,
 	.len = sizeof(hmaccalc_hmackey) - 1,
-	.subdir = "hmaccalc",
+#ifdef CHECK_DIR
+	.checkdir = CHECK_DIR"/hmaccalc",
+#else
+	.checkdir = NULL,
+#endif
 };
 
 static void usage(char *name, int fipscheck)
@@ -138,7 +150,8 @@ static void usage(char *name, int fipscheck)
 	fprintf(stderr, "\t-k --key-file FILE\tUse HMAC key from given file\n");
 	fprintf(stderr, "\t-K --key KEY\t\tUse KEY as the HMAC key\n");
 	fprintf(stderr, "\t   --tag\t\tCreate a BSD-style checksum\n");
-	fprintf(stderr, "\t-b, -d, -P\t\tCompatibility hmaccalc options; ignored\n");
+	fprintf(stderr, "\t-d\t\t\tCheck directory for fipshmac; otherwise ignored\n");
+	fprintf(stderr, "\t-b, -P\t\t\tCompatibility hmaccalc options; ignored\n");
 	fprintf(stderr, "\t   --help\t\tPrint this help text\n");
 	fprintf(stderr, "\t-v --version\t\tShow version\n");
 }
@@ -214,7 +227,7 @@ static int load_file(const char *filename, uint8_t **memory, uint32_t *size)
 	while ((rdbytes = read(fd, buffer + offset, buffer_size - offset)) != 0) {
 		if (rdbytes < 0) {
 			fprintf(stderr, "Error reading file %s: %s\n", filename,
-			        strerror((int)rdbytes));
+			        strerror(errno));
 			ret = -EIO;
 			goto out;
 		}
@@ -245,6 +258,8 @@ static int load_file(const char *filename, uint8_t **memory, uint32_t *size)
 
 	*memory = buffer;
 	*size = (uint32_t)offset;
+
+	close(fd);
 	return 0;
 
 out:
@@ -341,6 +356,17 @@ out:
 	return ret;
 }
 
+/*
+ * GCC v8.1.0 introduced -Wstringop-truncation but it is not smart enough to
+ * find that cursor string will be NULL-terminated after all paste() calls and
+ * warns with:
+ * error: 'strncpy' destination unchanged after copying no bytes [-Werror=stringop-truncation]
+ * error: 'strncpy' output truncated before terminating nul copying 5 bytes from a string of the same length [-Werror=stringop-truncation]
+ */
+#pragma GCC diagnostic push
+#if GCC_VERSION >= 80100
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
+#endif
 static char *paste(char *dst, const char *src, size_t size)
 {
 	strncpy(dst, src, size);
@@ -353,7 +379,7 @@ static char *paste(char *dst, const char *src, size_t size)
  * return: NULL when malloc failed, a pointer that the caller must free
  * otherwise.
  */
-static char *get_hmac_file(const char *filename, const char *subdir)
+static char *get_hmac_file(const char *filename, const char *checkdir)
 {
 	size_t i, filelen, pathlen, namelen, basenamestart = 0;
 	size_t prefixlen = strlen(CHECK_PREFIX);
@@ -371,12 +397,7 @@ static char *get_hmac_file(const char *filename, const char *subdir)
 	}
 
 	namelen = filelen - basenamestart;
-#ifdef CHECK_DIR
-	pathlen = strlen(CHECK_DIR"/") + strlen(subdir) + 1;
-#else
-	(void)subdir; // avoid parameter unused warning
-	pathlen = basenamestart;
-#endif
+	pathlen = checkdir ? strlen(checkdir) + 1 : basenamestart;
 
 	checkfile = malloc(pathlen + namelen + prefixlen + 1 /* "." */ +
 		suffixlen + 1 /* null character */);
@@ -384,24 +405,23 @@ static char *get_hmac_file(const char *filename, const char *subdir)
 		return NULL;
 
 	cursor = checkfile;
-#ifdef CHECK_DIR
-	cursor = paste(cursor, CHECK_DIR"/", strlen(CHECK_DIR"/"));
-	cursor = paste(cursor, subdir, strlen(subdir));
-	cursor = paste(cursor, "/", 1);
-#else
-	if (pathlen > 0)
+	if (checkdir) {
+		cursor = paste(cursor, checkdir, strlen(checkdir));
+		cursor = paste(cursor, "/", 1);
+	} else if (pathlen > 0)
 		cursor = paste(cursor, filename, pathlen);
-#endif
+
 	cursor = paste(cursor, CHECK_PREFIX, prefixlen);
 	cursor = paste(cursor, filename + basenamestart, namelen);
 	cursor = paste(cursor, "."CHECK_SUFFIX, 1 + suffixlen);
 	strncpy(cursor, "\0", 1);
 	return checkfile;
 }
+#pragma GCC diagnostic pop /* -Wstringop-truncation */
 
 static int hash_files(const struct hash_params *params,
 		      char *filenames[], uint32_t files,
-		      int fipshmac, int just_print)
+		      int fipshmac, const char *checkdir, int just_print)
 {
 	struct kcapi_handle *handle;
 	const char *hashname = params->name.kcapiname;
@@ -430,9 +450,7 @@ static int hash_files(const struct hash_params *params,
 			const char *filename = filenames[i];
 
 			if (fipshmac) {
-				char *outfile = get_hmac_file(filenames[i],
-				                              params->key.subdir);
-
+				char *outfile = get_hmac_file(filenames[i], checkdir);
 				if (!outfile) {
 					fprintf(stderr,
 						"Cannot create HMAC file name\n");
@@ -514,8 +532,11 @@ static int process_checkfile(const struct hash_params *params,
 		uint32_t i;
 		uint32_t bsd_style = 0; // >0 if --tag formatted style
 
+		if (linelen == 0)
+			break;
+
 		/* remove trailing CR and reduce buffer length */
-		for (i = linelen; i > 0; i--) {
+		for (i = linelen - 1; i > 0; i--) {
 			if (!isprint(buf[i])) {
 				buf[i] = '\0';
 				linelen--;
@@ -523,7 +544,7 @@ static int process_checkfile(const struct hash_params *params,
 				break;
 		}
 
-		for (i = 0; i < linelen; i++) {
+		for (i = 1; i < linelen; i++) {
 			/*
 			 * Check for BSD-style separator between file name and
 			 * hash value.
@@ -693,11 +714,11 @@ static int fipscheck_self(const struct hash_params *params_bin,
 		}
 
 		if (mode == SELFCHECK_PRINT_SELF) {
-			ret = hash_files(params_bin, names, 1, 0, 1);
+			ret = hash_files(params_bin, names, 1, 0, NULL, 1);
 			goto out;
 		}
 
-		checkfile = get_hmac_file(selfname, params_bin->key.subdir);
+		checkfile = get_hmac_file(selfname, params_bin->key.checkdir);
 		if (!checkfile) {
 			ret = -ENOMEM;
 			goto out;
@@ -731,13 +752,13 @@ static int fipscheck_self(const struct hash_params *params_bin,
 		strncpy(selfname, info.dli_fname, (sizeof(selfname) - 1));
 
 		if (mode == SELFCHECK_PRINT_LIB) {
-			ret = hash_files(params_lib, names, 1, 0, 1);
+			ret = hash_files(params_lib, names, 1, 0, NULL, 1);
 			goto out;
 		}
 
 		if (checkfile)
 			free(checkfile);
-		checkfile = get_hmac_file(selfname, params_lib->key.subdir);
+		checkfile = get_hmac_file(selfname, params_lib->key.checkdir);
 		if (!checkfile) {
 			ret = -ENOMEM;
 			goto out;
@@ -780,6 +801,7 @@ int main(int argc, char *argv[])
 
 	char *checkfile = NULL;
 	const char *targetfile = NULL;
+	const char *checkdir = NULL;
 	uint8_t *hmackey_alloc = NULL;
 	uint8_t *hmackey_mmap = NULL;
 	int opt_index = 0;
@@ -841,10 +863,12 @@ int main(int argc, char *argv[])
 	}
 	basen = basename(basec);
 
+	opterr = 0;
 	if (getopt_long(argc, argv, opts_name_short, opts_name, &opt_index) == 'n')
 		basen = optarg;
 	else
-		opt_index = 0;
+		optind = 1;
+	opterr = 1;
 
 	params_self = &PARAMS_SELF_FIPSCHECK;
 	if (0 == strncmp(basen, "sha256sum", 9)) {
@@ -1034,8 +1058,10 @@ int main(int argc, char *argv[])
 				version(argv[0]);
 				ret = 0;
 				goto out;
-			case 'b':
 			case 'd':
+				checkdir = optarg;
+				break;
+			case 'b':
 			case 'P':
 				/* Compatibility options, just ignore */
 				break;
@@ -1089,7 +1115,7 @@ int main(int argc, char *argv[])
 		targetfile = argv[optind];
 		if (checkfile)
 			free(checkfile);
-		checkfile = get_hmac_file(targetfile, params.key.subdir);
+		checkfile = get_hmac_file(targetfile, params.key.checkdir);
 		if (!checkfile) {
 			ret = 1;
 			goto out;
@@ -1099,7 +1125,7 @@ int main(int argc, char *argv[])
 
 	if (!checkfile)
 		ret = hash_files(&params, argv + optind, (argc - optind),
-		                 fipshmac, 0);
+		                 fipshmac, checkdir, 0);
 	else if (optind == argc)
 		ret = process_checkfile(&params, checkfile, targetfile, loglevel);
 	else {
